#! /bin/sh
#
# Author: Juergen Vigna <juergen.vigna@wuerth-phoenix.com>
# Version: 1.0.0
# Licence: GPLV2
# Params: (-f rrd-file|-H host -s <service>) -F <from-time> -T <to-time> -d <data-source-number>
# Description:
#   This scripts gets the average values of yesterday, 2 days ago, the last 7 days
#   and the last 30 days of the time-period and data-source-number specified out of
#   an rrd-file in the format yesterday-value:2da-value:last-week-value:7d-value:30d-value
#

TMPFILE_BASE="/var/tmp/rrd_get_values_$$"
DEBUGFILE="/var/tmp/rrd_get_values_$$_debug"
RRDPATH=/var/log/nagios/perfdata

trap 'rm -f ${TMPFILE_BASE}_* exit 10' 1 2 15
trap 'rm -f ${TMPFILE_BASE}_*' 0

VERSION="1.0.0"
PROGNAME=`basename $0`
PROGPATH=`echo $0 | /bin/sed -e 's,[\\/][^\\/][^\\/]*$,,'`

PATH=$PATH:/bin:/usr/bin:/usr/local/bin
export PATH

print_header()  {
        echo "$PROGNAME - $VERSION"
        echo ""
        echo "This program gets performance data from a Service RRD file in the form:"
        echo "  <yesterday>:<day before yesterday>:<last week):<last 7 days>:<last 30 days>"
        echo ""

}

print_usage() {
        echo "Usage: $PROGNAME [-D] [-a] [-d <ds-number>] (-f <rrd-file>|-H hostname -s service-rrd)"
	echo "                 [-F <from-time>] [-T <to-time>]"
	echo "       $PROGNAME -l -H hostname ... to list all service-rrd for this host"
	echo ""
        echo "       -h, --help    : this help"
        echo "       -V, -version  : program version"
        echo "       -D            : output debug informations"
        echo "       -f <rrd-file> : rrd-file-name with full path"
        echo "       -H            : Hostname as defined inside Nagios Configuration"
        echo "       -s            : service rrd name -l to see all available source files for a host"
        echo "       -F            : from time in the format HH:MM:SS (default: 00:00:00)"
        echo "       -T            : to time in the format HH:MM:SS (default: 23:59:59)"
        echo "       -d            : data source number inside the rrd (default: 1)"
        echo "       -a            : weekend are excluded by default use this to include them"
        echo
        echo "The time format is HH (00-23), MM (00-59), SS (00-59)"
        echo
}

print_help() {
        print_header
        print_usage
        echo ""
}

# ---------------------------------------------------------------------------
# INITIALIZE VARIABLES
# ---------------------------------------------------------------------------

DEBUG=0
ALLDAYS=0
LISTSERVICES=0
RRDFILE=""
HOST=""
SERVICE=""
FROMTIME="00:00:00"
TOTIME="23:59:59"

# ---------------------------------------------------------------------------
# OPTION HANDLING
# ---------------------------------------------------------------------------

while [ -n "$1" ]
do
    opt=`echo "$1" | cut -d= -f1`
    optarg=`echo "$1" | cut -d= -f2-`
    if [ -z "$optarg" ]
    then
        echo "Argument not specified for extended option $1"
        exit 3
    fi
    if [ "$optarg" = "$opt" ]
    then
        optarg="$2"
    fi
    case "$opt" in
        --help)
            print_help
            exit 0
            ;;
        -h)
            print_help
            exit 0
            ;;
        --version)
            print_revision $PROGNAME $VERSION
            exit 0
            ;;
        -V)
            print_revision $PROGNAME $VERSION
            exit 0
            ;;
        --file | -f)
            RRDFILE=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --host | -H)
            HOST=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --service | -s)
            SERVICE=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --from-time | -F)
            FROMTIME=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --to-time | -T)
            TOTIME=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --debug | -D)
            DEBUG=1
            ;;
        --ds | -d)
            DS=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --include-weekend | -a)
            ALLDAYS=1
            ;;
        --list | -l)
            LISTSERVICES=1
            ;;
        *)
            print_usage
            echo "Unknown argument: $1"
            exit 3
            ;;
    esac
    shift
done

if [ $LISTSERVICES -eq 1 ]
then
    if [ -z "$HOST" ]
    then
        print_usage
        echo "You cannot list service-rrd-files if you do not specify a hostname"
        exit 3
    fi
    if [ ! -d $RRDPATH/$HOST ]
    then
        print_usage
        echo "HOST -> $HOST seems not to exist inside $RRDPATH"
        exit 3
    fi
    echo
    echo "List service RRD names of host $HOST:"
    echo
    cd $RRDPATH/$HOST
    ls -1 *.rrd | sed -e 's/.rrd//g'
    echo
    exit 0
fi

if [ -z "$RRDFILE" ]
then
    if [ -z "$HOST" -o -z "$SERVICE" ]
    then
        print_usage
        echo "Missing RRDFILE parameter or HOST and SERVICE parameters"
        exit 3
    fi
    RRDFILE="$RRDPATH/$HOST/$SERVICE.rrd"
fi

dd=`dirname "$RRDFILE"`
nn=`basename "$RRDFILE"`
ff=`find "$dd" -name "$nn" | head -n 1`
RRDFILE="$ff"
if [ ! -f "$RRDFILE" ]
then
    print_usage
    echo "File $RRDPATH/$HOST/$SERVICE.rrd does not seem to exist please use -l to list service files"
    exit 3
fi

if [ -z "$FROMTIME" -o -z "$TOTIME" ]
then
	print_usage
	echo "Missing from/to time parameter"
	exit 3
fi
 
if [ -z "$DS" ]
then
	DS=1
fi

col=`expr $DS + 1`

# ---------------------------------------------------------------------------
# HELPER FUNCTIONS
# ---------------------------------------------------------------------------

#
# Function: is_weekday
# Parameters: number of day to compute
# Description:
#   Returns 1 "number day ago" is a weekday and 0 else
#
is_weekday() {
	day_of_week=`date +"%u" -d "$1 day ago"`
	if [ $day_of_week -ge 1 -a $day_of_week -le 5 ]
	then
		echo 1
	else
		echo 0
	fi
}

#
# Function: compute_values
# Parameters: fromday [today}
# Description:
#   Get an average value of all the data inside the datafiles specified by the params
#   If today is not specified then get only 1 day data file
#
compute_values() {
	from_day=$1
	if [ -z "$2" ]
	then
		to_day=$1
	else
		to_day=$2
	fi

	vv=0
	nn=0
	d1=$from_day
	while [ $d1 -le $to_day ]
	do
		wdflag=`is_weekday $d1`
		if [ $ALLDAYS -gt 0 -o $wdflag -gt 0 ]
		then
			v1=`cat ${TMPFILE_BASE}_$d1 | awk -v col=$col 'BEGIN{n=0;vv=0;}{if ($col != "nan") {vv=vv+$col;n=n+1;}}END{ if (n>0) { printf "%d",vv/n;} else { printf "nan";}}'`
			if [ "$v1" != "nan" ]
			then
				vv=`expr $vv + $v1`
				nn=`expr $nn + 1`
			fi
		elif [ $DEBUG -ne 0 ]
		then
			echo "Day $d1 excluded because it's a weekend day" >>$DEBUGFILE
		fi
		d1=`expr $d1 + 1`
	done
	if [ $nn -gt 0 ]
	then
		echo `date +'%s' -d "$from_day day ago"`:`expr $vv / $nn`
	else
		echo `date +'%s' -d "$from_day day ago"`:nan
	fi
}

# ---------------------------------------------------------------------------
# START MAIN PROGRAM
# ---------------------------------------------------------------------------
dd=1
while [ $dd -le 31 ]
do
	d1=`date +"%s" -d "$dd day ago $FROMTIME"`
	d2=`date +"%s" -d "$dd day ago $TOTIME"`
	rrdtool fetch $RRDFILE AVERAGE -r 3600 -s $d1 -e $d2 | tail -n +3 >${TMPFILE_BASE}_$dd
	if [ ! -s "${TMPFILE_BASE}_$dd" ]
	then
		echo "Error getting data from file ($rrdfile)"
		exit 3
	fi
	dd=`expr $dd + 1`
done

dd=1
wd=`is_weekday 1`
while [ $ALLDAYS -eq 0 -a $wd -eq 0 ]
do
	dd=`expr $dd + 1`
	wd=`is_weekday $dd`
done
v1=`compute_values $dd`
dlw=`expr $dd + 7`
dd=`expr $dd + 1`
wd=`is_weekday 2`
while [ $ALLDAYS -eq 0 -a $wd -eq 0 ]
do
	dd=`expr $dd + 1`
	wd=`is_weekday $dd`
done
v2=`compute_values $dd`
v3=`compute_values $dlw`
v4=`compute_values 1 8`
v5=`compute_values 1 31`

echo "$v1;$v2;$v3;$v4;$v5"
if [ $DEBUG -gt 0 ]
then
	cat $DEBUGFILE
fi
exit 0
